@@ -1,17 +1,11 @@
 #!/usr/bin/env python3
 
-# mypy: allow-any-generics
-
-from __future__ import print_function
-
 import itertools
 import os
 import re
 import sys
 
 import google.protobuf
-PR_V = str(google.protobuf.__version__)
-assert PR_V.split(".") >= "3.13.0".split("."), "python %s:%s: protobuf %s version not supported" % (sys.version, sys.executable, PR_V)
 
 from collections import defaultdict, namedtuple, OrderedDict
 from contextlib import contextmanager
@@ -19,12 +13,14 @@ from typing import (
     Any,
     DefaultDict,
     Dict,
+    Generic,
     Iterator,
     List,
     Optional,
     Set,
     Text,
     Tuple,
+    TypeVar,
     Union,
 )
 
@@ -41,7 +37,6 @@ from google.protobuf.descriptor_pb2 import (
 )
 from google.protobuf.message import Message
 
-from proto.github.com.gogo.protobuf.gogoproto import gogo_pb2
 from proto.rust import extensions_pb2
 
 # Proto type -> (RustType, ImplsEq, ImplsCopy))
@@ -133,7 +128,7 @@ WalkRet = Tuple[
     List[Tuple[List[Text], EnumDescriptorProto, SourceCodeLocation]],
     List[Tuple[List[Text], DescriptorProto, SourceCodeLocation]],
 ]
-ModTree = DefaultDict[Text, DefaultDict]
+ModTree = DefaultDict[Text, DefaultDict[Text, Any]]
 
 
 def camelcase(underscored: Text) -> Text:
@@ -152,7 +147,13 @@ class StringIO(object):
 
 
 class RustType(object):
-    def __init__(self, ctx: 'Context', proto_file: FileDescriptorProto, msg_type: DescriptorProto, field: FieldDescriptorProto) -> None:
+    def __init__(
+        self,
+        ctx: "Context",
+        proto_file: FileDescriptorProto,
+        msg_type: DescriptorProto,
+        field: FieldDescriptorProto,
+    ) -> None:
         self.ctx = ctx
         self.proto_file = proto_file
         self.field = field
@@ -274,8 +275,8 @@ class RustType(object):
             # they cannot actually be represented on the wire as null
             # rather they are represented as 0-value ("" or 0)
             return False
-        if self.field.options.HasExtension(gogo_pb2.nullable):
-            return self.field.options.Extensions[gogo_pb2.nullable]
+        if self.field.options.HasExtension(extensions_pb2.nullable_field):
+            return self.field.options.Extensions[extensions_pb2.nullable_field]
         return (
             not self.is_proto3 or self.field.type == FieldDescriptorProto.TYPE_MESSAGE
         )
@@ -286,10 +287,14 @@ class RustType(object):
 
     def can_be_packed(self) -> bool:
         # Return true if incoming messages could be packed on the wire
-        return self.field.label == FieldDescriptorProto.LABEL_REPEATED and self.wire_format() in (
-            "Varint",
-            "Fixed64",
-            "Fixed32",
+        return (
+            self.field.label == FieldDescriptorProto.LABEL_REPEATED
+            and self.wire_format()
+            in (
+                "Varint",
+                "Fixed64",
+                "Fixed32",
+            )
         )
 
     def should_serialize_packed(self) -> bool:
@@ -412,13 +417,13 @@ class RustType(object):
         elif self.field.type == FieldDescriptorProto.TYPE_STRING:
             return (
                 "&str",
-                'self.%s.as_ref().map(|ref s| s.as_str()).unwrap_or("")' % name,
+                'self.%s.as_deref().unwrap_or("")' % name,
             )
         elif self.field.type == FieldDescriptorProto.TYPE_BYTES:
             assert not (
                 self.is_blob() or self.is_grpc_slices() or self.is_lazy_bytes()
             ), "Can't generate get method for lazy field"
-            return "&[u8]", "self.%s.as_ref().map(|v| &**v).unwrap_or(&[])" % name
+            return "&[u8]", "self.%s.as_deref().unwrap_or(&[])" % name
         elif self.field.type == FieldDescriptorProto.TYPE_ENUM:
             return self.rust_type(), "self.%s.unwrap_or_default()" % name
         elif self.field.type == FieldDescriptorProto.TYPE_MESSAGE:
@@ -469,13 +474,13 @@ class RustType(object):
         rust_type = self.rust_type()
 
         if self.is_repeated():
-            return "::std::vec::Vec<%s>" % rust_type  # type: ignore[return-value]
+            return "::std::vec::Vec<%s>" % rust_type
         elif self.is_nullable() and self.is_boxed():
             return "::std::option::Option<::std::boxed::Box<%s>>" % str(rust_type)
         elif self.is_boxed():
-            return "::std::boxed::Box<%s>" % rust_type  # type: ignore[return-value]
+            return "::std::boxed::Box<%s>" % rust_type
         elif self.is_nullable():
-            return "::std::option::Option<%s>" % rust_type  # type: ignore[return-value]
+            return "::std::option::Option<%s>" % rust_type
         else:
             return str(rust_type)
 
@@ -515,6 +520,7 @@ def enum_err_if_default_or_unknown(enum: EnumDescriptorProto) -> bool:
         and enum.options.Extensions[extensions_pb2.err_if_default_or_unknown]
     )
 
+
 def enum_closed(enum: EnumDescriptorProto) -> bool:
     return (
         enum.options.HasExtension(extensions_pb2.closed_enum)
@@ -523,7 +529,7 @@ def enum_closed(enum: EnumDescriptorProto) -> bool:
 
 
 @contextmanager
-def block(ctx: 'CodeWriter', header: Text) -> Iterator[None]:
+def block(ctx: "CodeWriter", header: Text) -> Iterator[None]:
     ctx.write("%s {" % header)
     ctx.indentation += 1
     yield
@@ -532,7 +538,13 @@ def block(ctx: 'CodeWriter', header: Text) -> Iterator[None]:
 
 
 @contextmanager
-def field_iter(ctx: 'CodeWriter', var: Text, msg_name: Text, msg_type: DescriptorProto, field: FieldDescriptorProto) -> Iterator[None]:
+def field_iter(
+    ctx: "CodeWriter",
+    var: Text,
+    msg_name: Text,
+    msg_type: DescriptorProto,
+    field: FieldDescriptorProto,
+) -> Iterator[None]:
     typ = ctx.rust_type(msg_type, field)
 
     if typ.oneof:
@@ -562,7 +574,7 @@ def field_iter(ctx: 'CodeWriter', var: Text, msg_name: Text, msg_type: Descripto
     elif (
         field.type == FieldDescriptorProto.TYPE_ENUM
         and not typ.is_repeated()
-        and enum_err_if_default_or_unknown(ctx.ctx.find(field.type_name).typ)
+        and enum_err_if_default_or_unknown(ctx.ctx.find_enum(field.type_name).typ)
     ):
         # The default value (as considered by proto) doesn't appear in the generated enum and
         # doesn't map to .default(). All of the values that actually get generated need to get
@@ -592,7 +604,13 @@ def field_iter(ctx: 'CodeWriter', var: Text, msg_name: Text, msg_type: Descripto
 
 
 class CodeWriter(object):
-    def __init__(self, ctx: 'Context', proto_file: FileDescriptorProto, crate: Text, mod_parts: List[Text]) -> None:
+    def __init__(
+        self,
+        ctx: "Context",
+        proto_file: FileDescriptorProto,
+        crate: Text,
+        mod_parts: List[Text],
+    ) -> None:
         self.ctx = ctx
         self.proto_file = proto_file
         self.crate = crate
@@ -623,7 +641,9 @@ class CodeWriter(object):
         self.content.write(s)
         self.content.write("\n")
 
-    def write_line_broken_text_with_prefix(self, text_block: Text, prefix: Text) -> None:
+    def write_line_broken_text_with_prefix(
+        self, text_block: Text, prefix: Text
+    ) -> None:
         if not text_block:
             return
         for l in text_block.rstrip().split("\n"):
@@ -644,10 +664,17 @@ class CodeWriter(object):
         if sci_loc.trailing_comments is not None:
             self.write_line_broken_text_with_prefix(sci_loc.trailing_comments, "///")
 
-    def rust_type(self, msg_type: DescriptorProto, field: FieldDescriptorProto) -> RustType:
+    def rust_type(
+        self, msg_type: DescriptorProto, field: FieldDescriptorProto
+    ) -> RustType:
         return RustType(self.ctx, self.proto_file, msg_type, field)
 
-    def gen_closed_enum(self, name: Text, enum_variants: List[Tuple[int, EnumValueDescriptorProto]], scl: SourceCodeLocation) -> None:
+    def gen_closed_enum(
+        self,
+        name: Text,
+        enum_variants: List[Tuple[int, EnumValueDescriptorProto]],
+        scl: SourceCodeLocation,
+    ) -> None:
 
         # Generate a closed enum
         self.write_comments(self.source_code_info_by_scl.get(tuple(scl)))
@@ -662,7 +689,7 @@ class CodeWriter(object):
         self.write("#[repr(i32)]")
         with block(self, "pub enum " + name):
             for idx, value in enum_variants:
-                vfn = EnumDescriptorProto.VALUE_FIELD_NUMBER  # type: ignore
+                vfn = EnumDescriptorProto.VALUE_FIELD_NUMBER
                 self.write_comments(
                     self.source_code_info_by_scl.get(tuple(scl + [vfn, idx]))
                 )
@@ -713,7 +740,12 @@ class CodeWriter(object):
                     for _, value in enum_variants:
                         self.write('%s::%s => "%s",' % (name, value.name, value.name))
 
-    def gen_open_enum(self, name: Text, enum_variants: List[Tuple[int, EnumValueDescriptorProto]], scl: SourceCodeLocation) -> None:
+    def gen_open_enum(
+        self,
+        name: Text,
+        enum_variants: List[Tuple[int, EnumValueDescriptorProto]],
+        scl: SourceCodeLocation,
+    ) -> None:
 
         closed_name = name + "_Closed"
 
@@ -729,7 +761,7 @@ class CodeWriter(object):
         self.write("pub struct %s(i32);" % name)
         with block(self, "impl " + name):
             for idx, value in enum_variants:
-                vfn = EnumDescriptorProto.VALUE_FIELD_NUMBER  # type: ignore
+                vfn = EnumDescriptorProto.VALUE_FIELD_NUMBER
                 self.write_comments(
                     self.source_code_info_by_scl.get(tuple(scl + [vfn, idx]))
                 )
@@ -812,7 +844,9 @@ class CodeWriter(object):
 
         self.gen_closed_enum(closed_name, enum_variants, scl)
 
-    def gen_enum(self, path: List[Text], enum_type: EnumDescriptorProto, scl: SourceCodeLocation) -> None:
+    def gen_enum(
+        self, path: List[Text], enum_type: EnumDescriptorProto, scl: SourceCodeLocation
+    ) -> None:
         assert self.indentation == 0
         name = "_".join(path + [enum_type.name])
         if enum_err_if_default_or_unknown(enum_type):
@@ -825,7 +859,9 @@ class CodeWriter(object):
         else:
             self.gen_open_enum(name, list(enumerate(enum_type.value)), scl)
 
-    def gen_msg(self, path: List[Text], msg_type: DescriptorProto, scl: SourceCodeLocation) -> None:
+    def gen_msg(
+        self, path: List[Text], msg_type: DescriptorProto, scl: SourceCodeLocation
+    ) -> None:
         assert self.indentation == 0
         name = "_".join(path + [msg_type.name])
 
@@ -837,9 +873,7 @@ class CodeWriter(object):
             if oneof.name in RESERVED_KEYWORDS:
                 oneof.name = oneof.name + "_"
 
-        oneof_fields: DefaultDict[Text, List[FieldDescriptorProto]] = defaultdict(
-            list
-        )
+        oneof_fields: DefaultDict[Text, List[FieldDescriptorProto]] = defaultdict(list)
 
         derives = ["Clone", "Debug", "PartialEq"]
         if self.derive_serde:
@@ -856,7 +890,7 @@ class CodeWriter(object):
         self.write("#[derive(%s)]" % ", ".join(sorted(derives)))
         with block(self, "pub struct " + name):
             for idx, field in enumerate(msg_type.field):
-                ffn = DescriptorProto.FIELD_FIELD_NUMBER  # type: ignore
+                ffn = DescriptorProto.FIELD_FIELD_NUMBER
                 self.write_comments(
                     self.source_code_info_by_scl.get(tuple(scl + [ffn, idx]))
                 )
@@ -908,10 +942,7 @@ class CodeWriter(object):
                             "pub fn take_%s(&mut self) -> ::std::vec::Vec<%s>"
                             % (field.name, typ.rust_type()),
                         ):
-                            self.write(
-                                "::std::mem::replace(&mut self.%s, ::std::vec::Vec::new())"
-                                % field.name
-                            )
+                            self.write("::std::mem::take(&mut self.%s)" % field.name)
 
                         with block(
                             self,
@@ -948,7 +979,9 @@ class CodeWriter(object):
                             ):
                                 self.write(return_expr)
 
-                        if not (typ.is_blob() or typ.is_grpc_slices() or typ.is_lazy_bytes()):
+                        if not (
+                            typ.is_blob() or typ.is_grpc_slices() or typ.is_lazy_bytes()
+                        ):
                             # It's hard to make this make sense, so let's not generate `get` method for lazy things.
                             return_type, return_expr = typ.get_method()
                             with block(
@@ -991,7 +1024,9 @@ class CodeWriter(object):
 
                         self.write("let mut %s_size = 0;" % field.name)
                         with field_iter(self, "val", name, msg_type, field):
-                            self.write("let l = ::pb_jelly::Message::compute_size(val);")
+                            self.write(
+                                "let l = ::pb_jelly::Message::compute_size(val);"
+                            )
                             if not typ.should_serialize_packed():
                                 self.write(
                                     "%s_size += ::pb_jelly::wire_format::serialized_length(%s);"
@@ -1046,7 +1081,9 @@ class CodeWriter(object):
                         with block(self, "if !self.%s.is_empty()" % field.name):
                             self.write("let mut size = 0;")
                             with field_iter(self, "val", name, msg_type, field):
-                                self.write("size += ::pb_jelly::Message::compute_size(val);")
+                                self.write(
+                                    "size += ::pb_jelly::Message::compute_size(val);"
+                                )
                             self.write(
                                 "::pb_jelly::wire_format::write(%s, ::pb_jelly::wire_format::Type::LengthDelimited, w)?;"
                                 % field.number
@@ -1060,7 +1097,9 @@ class CodeWriter(object):
                                 % (field.number, typ.wire_format())
                             )
                         if typ.is_length_delimited():
-                            self.write("let l = ::pb_jelly::Message::compute_size(val);")
+                            self.write(
+                                "let l = ::pb_jelly::Message::compute_size(val);"
+                            )
                             self.write("::pb_jelly::varint::write(l as u64, w)?;")
                         self.write("::pb_jelly::Message::serialize(val, w)?;")
                 if msg_type.options.Extensions[extensions_pb2.preserve_unrecognized]:
@@ -1075,7 +1114,9 @@ class CodeWriter(object):
                     extensions_pb2.preserve_unrecognized
                 ]
                 if preserve_unrecognized:
-                    self.write("let mut unrecognized = ::pb_jelly::Unrecognized::default();")
+                    self.write(
+                        "let mut unrecognized = ::pb_jelly::Unrecognized::default();"
+                    )
 
                 for oneof in msg_type.oneof_decl:
                     if not oneof_nullable(oneof):
@@ -1084,16 +1125,14 @@ class CodeWriter(object):
                             "let mut oneof_%s: ::std::option::Option<%s> = None;"
                             % (oneof.name, oneof_typ)
                         )
-                err_if_default_field_names: OrderedDict[Text, None] = (
-                    OrderedDict()
-                )
+                err_if_default_field_names: OrderedDict[Text, None] = OrderedDict()
                 for field in msg_type.field:
                     typ = self.rust_type(msg_type, field)
                     if (
                         field.type == FieldDescriptorProto.TYPE_ENUM
                         and not typ.is_repeated()
                     ):
-                        enum_type = self.ctx.find(field.type_name).typ
+                        enum_type = self.ctx.find_enum(field.type_name).typ
                         if enum_err_if_default_or_unknown(enum_type) and not typ.oneof:
                             self.write(
                                 "let mut %s: ::std::option::Option<%s> = None;"
@@ -1252,7 +1291,13 @@ class CodeWriter(object):
                     self.write("unrecognized.serialize(&mut self._unrecognized)?;")
                 self.write("Ok(())")
 
-    def gen_msg_descriptor(self, path: List[Text], desc_proto: DescriptorProto, package: Optional[Text], scl: SourceCodeLocation) -> None:
+    def gen_msg_descriptor(
+        self,
+        path: List[Text],
+        desc_proto: DescriptorProto,
+        package: Optional[Text],
+        scl: SourceCodeLocation,
+    ) -> None:
         assert self.indentation == 0
 
         name = "_".join(path + [desc_proto.name])
@@ -1266,38 +1311,49 @@ class CodeWriter(object):
 def walk(proto: FileDescriptorProto) -> WalkRet:
     enums, messages = [], []
 
-    def _walk(proto: ProtoTypes, parents: List[Text], scl_prefix: SourceCodeLocation) -> None:
+    def _walk(
+        proto: ProtoTypes, parents: List[Text], scl_prefix: SourceCodeLocation
+    ) -> None:
         if isinstance(proto, EnumDescriptorProto):
             enums.append((parents, proto, scl_prefix))
         elif isinstance(proto, DescriptorProto):
             messages.append((parents, proto, scl_prefix))
 
             for i, nested_enum in enumerate(proto.enum_type):
-                etfn = DescriptorProto.ENUM_TYPE_FIELD_NUMBER  # type: ignore
+                etfn = DescriptorProto.ENUM_TYPE_FIELD_NUMBER
                 _walk(nested_enum, parents + [proto.name], scl_prefix + [etfn, i])
 
             for i, nested_message in enumerate(proto.nested_type):
-                ntfn = DescriptorProto.NESTED_TYPE_FIELD_NUMBER  # type: ignore
+                ntfn = DescriptorProto.NESTED_TYPE_FIELD_NUMBER
                 _walk(nested_message, parents + [proto.name], scl_prefix + [ntfn, i])
         elif isinstance(proto, FileDescriptorProto):
             for i, enum_type in enumerate(proto.enum_type):
-                etfn = FileDescriptorProto.ENUM_TYPE_FIELD_NUMBER  # type: ignore
+                etfn = FileDescriptorProto.ENUM_TYPE_FIELD_NUMBER
                 _walk(enum_type, parents, scl_prefix + [etfn, i])
 
             for i, message_type in enumerate(proto.message_type):
-                mtfn = FileDescriptorProto.MESSAGE_TYPE_FIELD_NUMBER  # type: ignore
+                mtfn = FileDescriptorProto.MESSAGE_TYPE_FIELD_NUMBER
                 _walk(message_type, parents, scl_prefix + [mtfn, i])
 
     _walk(proto, [], [])
     return enums, messages
 
 
-class ProtoType(object):
-    def __init__(self, ctx: 'Context', proto_file: FileDescriptorProto, path: List[Text], typ: Message) -> None:
+M = TypeVar("M", DescriptorProto, EnumDescriptorProto)
+
+
+class ProtoType(Generic[M]):
+    def __init__(
+        self,
+        ctx: "Context",
+        proto_file: FileDescriptorProto,
+        path: List[Text],
+        typ: M,
+    ) -> None:
         self.ctx = ctx
         self.proto_file = proto_file
         self.path = path  # inside proto file
-        self.typ: Any = typ
+        self.typ: M = typ
         self.crate, self.mod_parts = ctx.crate_from_proto_filename(proto_file.name)
 
     def __repr__(self) -> str:
@@ -1338,7 +1394,9 @@ Impls = namedtuple("Impls", ["Eq", "Copy"])
 
 class Context(object):
     def __init__(self, crate_per_dir: bool, prefix_to_clear: Text) -> None:
-        self.proto_types: Dict[Text, ProtoType] = {}
+        self.proto_types: Dict[
+            Text, Union[ProtoType[DescriptorProto], ProtoType[EnumDescriptorProto]]
+        ] = {}
 
         # Set iteration order is nondeterministic, but this is ok, because we can
         # emit crates in any order
@@ -1356,7 +1414,13 @@ class Context(object):
         # of how to split the crates apart.
         self.prefix_to_clear = prefix_to_clear
 
-    def calc_impls(self, proto_file: FileDescriptorProto, crate: Text, msg_type: DescriptorProto, fq_msg: Tuple[Text, Text]) -> None:
+    def calc_impls(
+        self,
+        proto_file: FileDescriptorProto,
+        crate: Text,
+        msg_type: DescriptorProto,
+        fq_msg: Tuple[Text, Text],
+    ) -> None:
         # Determine if each message can implement Eq / Ord Copy traits. Using Dynamic Programming.
         # See if it's cached
         if fq_msg in self.impls_by_msg:
@@ -1399,7 +1463,10 @@ class Context(object):
                 (msg_impls_eq, msg_impls_copy) = (False, False)  # Blob is not eq/copy
                 self.extra_crates[crate].add("blob_pb")
             # If we use a Bytes type
-            elif typ == FieldDescriptorProto.TYPE_BYTES and field.options.Extensions[extensions_pb2.zero_copy]:
+            elif (
+                typ == FieldDescriptorProto.TYPE_BYTES
+                and field.options.Extensions[extensions_pb2.zero_copy]
+            ):
                 (msg_impls_eq, msg_impls_copy) = (False, False)
                 self.extra_crates[crate].add("bytes")
             elif typ in PRIMITIVE_TYPES:
@@ -1410,6 +1477,7 @@ class Context(object):
             elif typ == FieldDescriptorProto.TYPE_ENUM:
                 pass  # Enums are Eq / Copy
             elif typ == FieldDescriptorProto.TYPE_MESSAGE:
+                assert isinstance(field_type.typ, DescriptorProto)
                 field_fq_msg = (
                     field_type.proto_file.name,
                     "_".join(field_type.path + [field_type.typ.name]),
@@ -1445,22 +1513,40 @@ class Context(object):
             crate, _ = self.crate_from_proto_filename(name)
             self.deps_map[crate]
 
-        for path, typ, _ in itertools.chain(enums, messages):
-            pt = ProtoType(self, proto_file, path, typ)
-            self.proto_types[pt.proto_name()] = pt
+        for enum_path, enum_typ, _ in enums:
+            enum_pt = ProtoType(self, proto_file, enum_path, enum_typ)
+            self.proto_types[enum_pt.proto_name()] = enum_pt
+
+        for path, typ, _ in messages:
+            msg_pt = ProtoType(self, proto_file, path, typ)
+            self.proto_types[msg_pt.proto_name()] = msg_pt
 
         for path, typ, _ in messages:
             fq_msg = (proto_file.name, "_".join(path + [typ.name]))
             crate, _ = self.crate_from_proto_filename(proto_file.name)
             self.calc_impls(proto_file, crate, typ, fq_msg)
 
-    def find(self, typename: Text) -> ProtoType:
+    def find_enum(self, typename: Text) -> ProtoType[EnumDescriptorProto]:
+        pt = self.find(typename)
+        assert isinstance(pt.typ, EnumDescriptorProto)
+        return pt
+
+    def find_msg(self, typename: Text) -> ProtoType[DescriptorProto]:
+        pt = self.find(typename)
+        assert isinstance(pt.typ, DescriptorProto)
+        return pt
+
+    def find(
+        self, typename: Text
+    ) -> Union[ProtoType[DescriptorProto], ProtoType[EnumDescriptorProto]]:
         if typename in self.proto_types:
             return self.proto_types[typename]
 
         raise RuntimeError("Could not find type by proto name: {}".format(typename))
 
-    def _set_boxed_if_recursive(self, visited: Set[Text], looking_for: Text, pt: ProtoType) -> bool:
+    def _set_boxed_if_recursive(
+        self, visited: Set[Text], looking_for: Text, pt: ProtoType[DescriptorProto]
+    ) -> bool:
         visited.add(pt.proto_name())
         any_field_boxed = False
         for field in pt.typ.field:
@@ -1471,7 +1557,7 @@ class Context(object):
                 need_box = False
                 if field.type_name not in visited:
                     need_box = self._set_boxed_if_recursive(
-                        visited, looking_for, self.find(field.type_name)
+                        visited, looking_for, self.find_msg(field.type_name)
                     )
                 if need_box or field.type_name == looking_for:
                     # We only box normal fields, not oneof variants
@@ -1493,11 +1579,13 @@ class Context(object):
 
     # This will recursively descend through a message and see if there any
     # recursive nesting.  In those cases, it will automatically box the fields.
-    def set_boxed_if_recursive(self, pt: ProtoType) -> None:
+    def set_boxed_if_recursive(self, pt: ProtoType[DescriptorProto]) -> None:
         visited: Set[Text] = set()
         self._set_boxed_if_recursive(visited, pt.proto_name(), pt)
 
-    def get_lib_and_mod_rs(self, mod_tree: ModTree, derive_serde: bool) -> Iterator[Tuple[Text, Text]]:
+    def get_lib_and_mod_rs(
+        self, mod_tree: ModTree, derive_serde: bool
+    ) -> Iterator[Tuple[Text, Text]]:
         for crate, deps in self.deps_map.items():
             librs = CodeWriter(None, None, None, None)  # type: ignore
             librs.write("#[macro_use]")
@@ -1507,18 +1595,16 @@ class Context(object):
                 librs.write("extern crate serde;")
             librs.write("")
 
-            def mod_tree_dfs(mod_prefix_path: Text, sub_mod_tree: ModTree) -> Iterator[Tuple[Text, Text]]:
+            def mod_tree_dfs(
+                mod_prefix_path: Text, sub_mod_tree: ModTree
+            ) -> Iterator[Tuple[Text, Text]]:
                 if not sub_mod_tree:
                     return
 
                 filename = mod_prefix_path + "/mod.rs"
                 content = "\n".join(
                     ["// @" + "generated, do not edit", ""]
-                    + [
-                        # TODO(ilevkivskyi): mypy reports a str/bytes issue.
-                        "pub mod %s;" % mod  # type: ignore[misc]
-                        for mod in sorted(sub_mod_tree.keys())
-                    ]
+                    + ["pub mod %s;" % mod for mod in sorted(sub_mod_tree.keys())]
                     + [""]
                 )
                 yield filename, content
@@ -1529,7 +1615,7 @@ class Context(object):
                     ):
                         yield res
 
-            crate_mod_tree = mod_tree[crate]
+            crate_mod_tree: ModTree = mod_tree[crate]
             for mod_name, child_mod_tree in sorted(crate_mod_tree.items()):
                 librs.write("pub mod %s;" % mod_name)
 
@@ -1547,9 +1633,9 @@ class Context(object):
 
     def get_spec_toml_file(self, derive_serde: bool) -> Iterator[Tuple[Text, Text]]:
         for crate, deps in self.deps_map.items():
-            all_deps = ({"lazy_static", "pb-jelly"} | deps | self.extra_crates[crate]) - {
-                "std"
-            }
+            all_deps = (
+                {"lazy_static", "pb-jelly"} | deps | self.extra_crates[crate]
+            ) - {"std"}
             if derive_serde:
                 all_deps.update({"serde"})
             features = {u"serde": u'features=["serde_derive"]'}
@@ -1562,17 +1648,17 @@ class Context(object):
 
     def get_cargo_toml_file(self, derive_serde: bool) -> Iterator[Tuple[Text, Text]]:
         for crate, deps in self.deps_map.items():
-            all_deps = ({"lazy_static", "pb-jelly"} | deps | self.extra_crates[crate]) - {
-                "std"
-            }
+            all_deps = (
+                {"lazy_static", "pb-jelly"} | deps | self.extra_crates[crate]
+            ) - {"std"}
             if derive_serde:
                 all_deps.update({"serde"})
             features = {u"serde": u' features = ["serde_derive"]'}
             versions = {
                 u"lazy_static": u' version = "1.4.0" ',
-                u"pb-jelly": u' version = "0.0.7" ',
+                u"pb-jelly": u' version = "0.0.9" ',
                 u"serde": u' version = "1.0" ',
-                u"bytes": u' version = "1.0" '
+                u"bytes": u' version = "1.0" ',
             }
 
             deps_lines = []
@@ -1588,11 +1674,17 @@ class Context(object):
                     "{dep} = {{{fields}}}".format(dep=dep, fields=",".join(fields))
                 )
 
-            targets = CARGO_TOML_TEMPLATE.format(crate=crate, deps="\n".join(deps_lines))
+            targets = CARGO_TOML_TEMPLATE.format(
+                crate=crate, deps="\n".join(deps_lines)
+            )
             yield crate, targets
 
-    def crate_from_proto_filename(self, proto_filename: Text) -> Tuple[Text, List[Text]]:
-        filename = proto_filename.replace(self.prefix_to_clear, "").replace(".proto", "")
+    def crate_from_proto_filename(
+        self, proto_filename: Text
+    ) -> Tuple[Text, List[Text]]:
+        filename = proto_filename.replace(self.prefix_to_clear, "").replace(
+            ".proto", ""
+        )
         mod_parts_unsanitized = filename.split("/")
         mod_parts = [
             mod + "_" if mod in RESERVED_KEYWORDS else mod
@@ -1653,6 +1745,7 @@ RS_HEADER = "// @" + "generated, do not edit\n"
 LIB_RS_HEADER = """
 #![warn(rust_2018_idioms)]
 #![allow(clippy::float_cmp)]
+#![allow(clippy::module_inception)]
 #![allow(irrefutable_let_patterns)]
 #![allow(non_camel_case_types)]
 #![allow(non_snake_case)]
@@ -1664,7 +1757,14 @@ LIB_RS_HEADER = """
 
 """
 
-def generate_single_crate(ctx: Context, file_prefix: Text, file_to_generate: List[Text], response: plugin.CodeGeneratorResponse) -> None:
+
+def generate_single_crate(
+    ctx: Context,
+    file_prefix: Text,
+    file_to_generate: List[Text],
+    request: plugin.CodeGeneratorRequest,
+    response: plugin.CodeGeneratorResponse,
+) -> None:
     def new_mod_tree() -> ModTree:
         return defaultdict(new_mod_tree)
 
@@ -1754,12 +1854,16 @@ def generate_single_crate(ctx: Context, file_prefix: Text, file_to_generate: Lis
             output.content = cargo_toml_file
 
 
-def generate_code(request: plugin.CodeGeneratorRequest, response: plugin.CodeGeneratorResponse) -> None:
+def generate_code(
+    request: plugin.CodeGeneratorRequest, response: plugin.CodeGeneratorResponse
+) -> None:
     to_generate = list(request.file_to_generate)
 
     prefix_to_clear = ""
     if "prefix_to_clear" in request.parameter:
-        prefix_to_clear = [arg for arg in request.parameter.split() if "prefix_to_clear" in arg][0].split("=")[1]
+        prefix_to_clear = [
+            arg for arg in request.parameter.split() if "prefix_to_clear" in arg
+        ][0].split("=")[1]
 
     if "crate_per_dir" in request.parameter:
         files_by_dir: DefaultDict[Text, List[Text]] = defaultdict(list)
@@ -1772,15 +1876,15 @@ def generate_code(request: plugin.CodeGeneratorRequest, response: plugin.CodeGen
             ctx = Context(crate_per_dir=True, prefix_to_clear=prefix_to_clear)
             for proto_file in request.proto_file:
                 ctx.feed(proto_file, to_generate)
-            generate_single_crate(ctx, file_prefix, to_generate, response)
+            generate_single_crate(ctx, file_prefix, to_generate, request, response)
     else:
         ctx = Context(crate_per_dir=False, prefix_to_clear=prefix_to_clear)
         for proto_file in request.proto_file:
             ctx.feed(proto_file, to_generate)
-        generate_single_crate(ctx, "", to_generate, response)
+        generate_single_crate(ctx, "", to_generate, request, response)
 
 
-if __name__ == "__main__":
+def main() -> None:
     # Read request message from stdin
     data = sys.stdin.buffer.read()
 
@@ -1799,3 +1903,7 @@ if __name__ == "__main__":
 
     # Write to stdout
     sys.stdout.buffer.write(output)
+
+
+if __name__ == "__main__":
+    main()
